<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://themeatelier.net
 * @since      1.0.0
 *
 * @package    Productify_Pro
 * @subpackage Productify_Pro/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Productify_Pro
 * @subpackage Productify_Pro/admin
 * @author     ThemeAtelier <themeatelierbd@gmail.com>
 */
class Productify_Pro_Admin {

	protected $suffix;
	private $plugin_name;
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->suffix      = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG || defined('WP_DEBUG') && WP_DEBUG ? '' : '.min';
		$this->plugin_name = $plugin_name;
		$this->version = $version;

		// Autoload system.
		spl_autoload_register(array($this, 'autoload'));

		PDT_Metaboxes::layout_metabox('ta_pdt_layouts');
		PDT_Metaboxes::option_metabox('ta_pdt_view_options');
		PDT_Metaboxes::shortcode_metabox('ta_pdt_display_shortcode');
		PDT_Settings::settings('ta_productify_settings');

		$active_plugins = get_option('active_plugins');
		foreach ($active_plugins as $active_plugin) {
			$_temp = strpos($active_plugin, 'productify-pro.php');
			if (false != $_temp) {
				add_filter('plugin_action_links_' . $active_plugin, array($this, 'add_plugin_action_links'));
			}
		}
	}

	public function add_plugin_action_links($links)
	{
		$new_links = array(
			sprintf('<a href="%s">%s</a>', admin_url('post-new.php?post_type=productify'), esc_html__('Add New', 'productify-pro')),
			sprintf('<a href="%s">%s</a>', admin_url('edit.php?post_type=productify'), esc_html__('Settings', 'productify-pro')),
		);
		return array_merge($new_links, $links);
	}

	/**
	 * Autoload class files on demand
	 *
	 * @param string $class requested class name.
	 * @since 2.2.0
	 */
	private function autoload($class)
	{
		$name = explode('_', $class);
		if (isset($name[1])) {
			$class_name       = strtolower($name[1]);
			$pdt_config_paths = array('views/', 'views/configs/settings/', 'views/configs/generator/');
			foreach ($pdt_config_paths as $ta_pdt_path) {
				$filename = plugin_dir_path(__FILE__) . '/' . $ta_pdt_path . 'class-pdt-' . $class_name . '.php';
				if (file_exists($filename)) {
					require_once $filename;
				}
			}
		}
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Productify_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Productify_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		 $current_screen        = get_current_screen();
		 $the_current_post_type = $current_screen->post_type;
		 if ('productify' == $the_current_post_type) {
			 wp_enqueue_style('productify-admin', PDT_URL . 'admin/assets/css/productify-pro-admin' . $this->suffix . '.css', array(), $this->version, 'all');
		 }

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Productify_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Productify_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		 $current_screen        = get_current_screen();
		 $the_current_post_type = $current_screen->post_type;
		 if ('productify' == $the_current_post_type) {
			 wp_enqueue_script('productify-admin', PDT_URL . 'admin/assets/js/productify-pro-admin' . $this->suffix . '.js', array('jquery'), $this->version, false);
		 }

	}

	/**
	 * Add productify admin columns.
	 *
	 * @since 2.0.0
	 * @return statement
	 */
	public function filter_productify_admin_column()
	{

		$admin_columns['cb']         = '<input type="checkbox" />';
		$admin_columns['title']      = esc_html__('Title', 'productify-pro');
		$admin_columns['shortcode']  = esc_html__('Shortcode', 'productify-pro');
		$admin_columns['pdt_layout'] = esc_html__('Layout', 'productify-pro');
		$admin_columns['date']       = esc_html__('Date', 'productify-pro');

		return $admin_columns;
	}

	/**
	 * Display admin columns for the productifys.
	 *
	 * @param mix    $column The columns.
	 * @param string $product_id The post ID.
	 * @return void
	 */
	public function display_productify_admin_fields($column, $product_id)
	{

		$pdt_layouts     = get_post_meta($product_id, 'ta_pdt_layouts', true);
		$productifys_types = isset($pdt_layouts['pdt_layout_preset']) ? $pdt_layouts['pdt_layout_preset'] : '';
		switch ($column) {
			case 'shortcode':
				$column_field = '<input  class="ta_pdt_input" style="width: 230px;padding: 4px 8px;cursor: pointer;" type="text" onClick="this.select();" readonly="readonly" value="[productify id=&quot;' . $product_id . '&quot;]"/> <div class="pdt-after-copy-text"><i class="far fa-check-circle"></i> Shortcode Copied to Clipboard! </div>';
				echo $column_field;
				break;
			case 'pdt_layout':
				$layout = ucwords(str_replace('_layout', ' ', $productifys_types));
				esc_html_e($layout, 'productify-pro');
				break;
		} // end switch.
	}

	/**
	 * If it is the plugins page.
	 *
	 * @since 2.2.0
	 * @access private
	 */
	private function is_plugins_screen()
	{
		return in_array(get_current_screen()->id, ['plugins', 'plugins-network']);
	}

}

/**
 * Productify dashboard capability.
 *
 * @return string
 */
function pdt_dashboard_capability()
{
	return apply_filters('pdt_dashboard_capability', 'manage_options');
}
