<?php

/**
 * The file functions.
 *
 * @package Productify_Pro
 * @subpackage Productify_Pro/public/helper
 *
 * @since 2.0.0
 */

/**
 * Post views helper method.
 *
 * @since 2.0.0
 */
class PDT_Functions {

	/**
	 * WP Version compare
	 *
	 * @param  mixed $version_to_compare version compare.
	 * @param  mixed $operator operator.
	 * @return statement
	 */
	public static function wp_version_compare( $version_to_compare, $operator = '>=' ) {
		if ( empty( $version_to_compare ) ) {
			return true;
		}
		global $wp_version;

		// Check if using WordPress version 3.7 or higher.
		return version_compare( $wp_version, $version_to_compare, $operator );
	}

	/**
	 * Post title character limit.
	 *
	 * @param string  $pdt_title The post title.
	 * @param integer $limit_length The length for the title.
	 * @param string  $pdt_after_string The string after title.
	 * @return statement
	 */
	public static function limit_post_title( $pdt_title, $limit_length, $pdt_after_string = '...' ) {
		return mb_strimwidth( $pdt_title, 0, $limit_length, apply_filters( 'pdt_product_title_ellipsis', $pdt_after_string ) );
	}

	/**
	 * Tag name to full tag conversion.
	 *
	 * @param array $meta_tag Tag option.
	 * @return string
	 */
	public static function short_tag_to_html( $meta_tag ) {
		$exclude_tag_string = '';
		foreach ( $meta_tag as $key => $value ) {
			$exclude_tag_string .= '<' . $value . '>,';
		}
		return $exclude_tag_string;
	}

	/**
	 * Limit the the text.
	 *
	 * @param mixed  $text The text you want to limit.
	 * @param int    $limit The number of words to display.
	 * @param string $ellipsis The ellipsis at the end of the text.
	 * @return statement
	 */
	public static function pdt_limit_text( $text, $limit, $ellipsis = '...' ) {
		$word_arr = explode( ' ', $text );
		if ( count( $word_arr ) > $limit ) {
			$words = implode( ' ', array_slice( $word_arr, 0, $limit ) ) . $ellipsis;
			return $words;
		} else {
			return $text;
		}
	}

	/**
	 * Characters Limit of the text.
	 *
	 * @param mixed  $text The text you want to limit.
	 * @param int    $limit The number of words to display.
	 * @param string $ellipsis The ellipsis at the end of the text.
	 * @return statement
	 */
	public static function limit_content_chr( $text, $limit, $ellipsis = '...' ) {
		$length = $limit;
		if ( strlen( $text ) < $length + 10 ) {
			return $text; // don't cut if too short.
		}
		$visible = substr( $text, 0, $length );
		return $visible . $ellipsis;
	}

	/**
	 * Allowed tags function of the Plugin.
	 *
	 * @since 2.0.0
	 * @return array allowed tags
	 */
	public static function allowed_tags() {
		$allowed_tags           = wp_kses_allowed_html( 'post' );
		$allowed_tags['iframe'] = array(
			'src'             => array(),
			'height'          => array(),
			'width'           => array(),
			'frameborder'     => array(),
			'allowfullscreen' => array(),
			'title'           => array(),
			'alt'             => array(),
		);

		$allowed_tags['style'] = array();

		return $allowed_tags;
	}

	/**
	 * Content function.
	 *
	 * @param array  $view_options Button options array.
	 * @param string $type Content type.
	 * @param  mixed  $product post.
	 * @return content
	 */
	public static function pdt_content( $view_options, $type, $product ) {
		$pdt_content_length_type      = isset( $view_options['pdt_content_length_type'] ) ? $view_options['pdt_content_length_type'] : 'words';
		$product_content_length       = isset( $view_options['pdt_content_limit'] ) ? $view_options['pdt_content_limit'] : '';
		$pdt_content_characters_limit = isset( $view_options['pdt_content_characters_limit'] ) ? $view_options['pdt_content_characters_limit'] : '';
		$product_content_ellipsis     = isset( $view_options['post_content_ellipsis'] ) ? $view_options['post_content_ellipsis'] : '';
		$pdt_strip_tags               = isset( $view_options['pdt_strip_tags'] ) ? $view_options['pdt_strip_tags'] : '';
		$pdt_allow_tag_name           = isset( $view_options['pdt_allow_tag_name'] ) ? $view_options['pdt_allow_tag_name'] : '';
		$allowed_tags                 = explode( ',', $pdt_allow_tag_name );

		$is_page_content = false;
		$is_page_content = apply_filters( 'pdt_strip_shortcode_in_page_content', $is_page_content );
		global $wp_embed;

		if ( 'excerpt' === $type ) {
			$pdt_product_content = $product->short_description;
		} elseif ( 'full_content' === $type ) {

			if ( $is_page_content ) {
				$product_content = apply_filters( 'ta_pdt_the_content', strip_shortcodes( $product->get_description() ) );
			} else {
				$product_content = apply_filters( 'ta_pdt_the_content', $product->get_description() );
			}
			if ( 'allow_some' === $pdt_strip_tags ) {
				$pdt_product_content = strip_tags( $product_content, self::short_tag_to_html( $allowed_tags ) );
			} elseif ( 'strip_all' === $pdt_strip_tags ) {
				$pdt_product_content = wp_strip_all_tags( $product_content );
			} else {
				$pdt_product_content = $product_content;
			}
		} else {
			if ( $is_page_content ) {
				$product_content = apply_filters( 'ta_pdt_the_content', strip_shortcodes( $product->get_description() ) );
			} else {
				$product_content = apply_filters( 'ta_pdt_the_content', $product->get_description() );
			}
			if ( 'characters' === $pdt_content_length_type ) {
				$_trimmed_content = ( 'strip_all' === $pdt_strip_tags ) ? wp_html_excerpt( $product_content, $pdt_content_characters_limit, $product_content_ellipsis ) : self::limit_content_chr( $product_content, $pdt_content_characters_limit, $product_content_ellipsis );
			} else {
				$_trimmed_content = self::pdt_limit_text( $product_content, $product_content_length, $product_content_ellipsis );
			}
			if ( 'allow_some' === $pdt_strip_tags ) {
				$pdt_product_content = strip_tags( $_trimmed_content, self::short_tag_to_html( $allowed_tags ) );
			} elseif ( 'strip_all' === $pdt_strip_tags ) {
				$pdt_product_content = wp_strip_all_tags( $_trimmed_content );
			} else {
				$pdt_product_content = $_trimmed_content;
			}
			$pdt_product_content = force_balance_tags( $pdt_product_content );
		}
		$pdt_product_content = do_shortcode( $wp_embed->autoembed( $pdt_product_content ) );
		return $pdt_product_content;
	}

	/**
	 * Get the ID by the Image URL.
	 *
	 * @param mixed $url The image url.
	 * @param int   $product_id The image url.
	 * @return statement
	 */
	public static function pdt_image_id_by_url( $url, $product_id = null ) {
		global $wpdb;

		// If the URL is auto-generated thumbnail, remove the sizes and get the URL of the original image.
		$url       = preg_replace( '/-\d+x\d+(?=\.(png|jp(e|g|eg)|gif|bmp|ico|webp|svg)$)/i', '', $url );
		$cache_key = 'pdt_content_img_url_to_id' . $product_id;
		$img_cache = wp_cache_get( $cache_key );
		if ( false === $img_cache ) {
			$image = $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE guid=%s;", $url ) );
			wp_cache_set( $cache_key, $img_cache );
		}
		if ( ! empty( $image ) ) {
			return $image[0];
		}
		return false;
	}
	/**
	 * Thumb alter text
	 *
	 * @param integer $slide_id The slide/post ID.
	 *
	 * @return string
	 */
	public static function pdt_thumb_alter_text( $slide_id ) {
		$image_id = get_post_thumbnail_id( $slide_id );
		$alt_text = get_post_meta( $image_id, '_wp_attachment_image_alt', true );
		return $alt_text;
	}

	/**
	 * Modify query params
	 *
	 * @param  array  $query_args query_args.
	 * @param  string $keyword keyword.
	 * @param  array  $custom_fields_array custom fields array.
	 * @param  string $orderby orderby.
	 * @param  string $order order.
	 * @param  array  $selected_term_list term_list.
	 * @param  int    $product_offset offset.
	 * @param  string $relation relation operator.
	 * @param  array  $product_in post in.
	 * @param  string $lang current lang.
	 * @return array
	 */
	public static function modify_query_params( $query_args, $keyword, $custom_fields_array, $orderby, $order, $selected_term_list, $product_offset, $relation, $product_in = array(), $lang = '' ) {
		if ( ! empty( $keyword ) ) {
			$query_args['s'] = $keyword;
			$product_offset  = 0;
		}
		if ( ! empty( $lang ) ) {
			$query_args['lang'] = $lang;
		}
		$query_args['post__in'] = $product_in;
		if ( ! empty( $custom_fields_array ) ) {
			$pdt_meta_query = array();
			foreach ( $custom_fields_array as $_custom_field ) {
				$c_field_key   = $_custom_field['custom_field_key'];
				$c_field_value = $_custom_field['custom_field_value'];
				if ( strpos( $c_field_value, ' ' ) !== false ) {
					$c_field_value    = explode( ' ', $c_field_value );
					$pdt_meta_query[] = array(
						'key'     => $c_field_key,
						'value'   => $c_field_value,
						'compare' => 'BETWEEN',
						'type'    => 'numeric',
					);
				} else {
					if ( strpos( $c_field_value, ',' ) !== false ) {
						$c_field_value = explode( ',', $c_field_value );
					}
					$pdt_meta_query[] = array(
						'key'     => $c_field_key,
						'compare' => 'IN',
						'value'   => $c_field_value,
					);
				}
			}
			if ( count( $pdt_meta_query ) > 1 ) {
				$pdt_meta_query['relation'] = 'AND';
			}
			$query_args['meta_query'] = $pdt_meta_query;
		}

		if ( ! empty( $orderby ) ) {
			$query_args['orderby'] = ( 'rand' === $orderby ) ? 'rand(' . get_transient( 'pdt_rand' ) . ')' : $orderby;
		}
		if ( ! empty( $order ) ) {
			$query_args['order'] = $order;
		}
		if ( ! empty( $selected_term_list ) ) {
			if ( count( $selected_term_list ) > 1 ) {
				$tax_settings['relation'] = $relation;
			}
			if ( is_array( $selected_term_list ) ) {
				foreach ( $selected_term_list as $key => $tax_type ) {
					$taxonomy = $tax_type['taxonomy'];
					$terms    = $tax_type['term_id'];
					if ( strpos( $terms, ',' ) !== false ) {
						$terms = explode( ',', $terms );
					}
					if ( $taxonomy && $terms ) {
						$tax_settings[] = array(
							'taxonomy'         => $taxonomy,
							'field'            => 'term_id',
							'terms'            => $terms,
							'operator'         => 'IN',
							'include_children' => false,
						);
					}
				}
				$query_args['tax_query'] = $tax_settings;
			}
			$product_offset = 0;
		}
		$query_args['offset'] = $product_offset;
		return $query_args;
	}

	/**
	 * Thumb Sized function
	 *
	 * @param array   $product_thumb_setting Thumbnails options array.
	 * @param integer $slide_id The slide/post ID.
	 * @param  bool    $is_attachment type attachment.
	 *
	 * @return string
	 */
	public static function pdt_sized_thumb( $product_thumb_setting, $slide_id, $is_attachment = false ) {
		$thumb_id                     = '';
		$image                        = '';
		$pdt_thumb_src_replace        = isset( $product_thumb_setting['pdt_thumb_src_replace'] ) ? $product_thumb_setting['pdt_thumb_src_replace'] : '';
		$product_featured_thumb_found = isset( $product_thumb_setting['post_featured_thumb_found'] ) ? $product_thumb_setting['post_featured_thumb_found'] : 'no_featured_img_found';
		$show_2x_image                = isset( $product_thumb_setting['load_2x_image'] ) ? $product_thumb_setting['load_2x_image'] : false;
		$image_resize_2x_url          = '';
		if ( 'even_featured_img_found' === $product_featured_thumb_found && is_array( $pdt_thumb_src_replace ) ) {
			$replace_thumb = self::pdt_thumb_replace( $product_thumb_setting, $slide_id );
			$thumb_id      = $replace_thumb['id'];
		} elseif ( ! has_post_thumbnail( $slide_id ) && 'no_featured_img_found' === $product_featured_thumb_found ) {
			$replace_thumb = self::pdt_thumb_replace( $product_thumb_setting, $slide_id );
			$thumb_id      = $replace_thumb['id'];
		} elseif ( has_post_thumbnail( $slide_id ) ) {
				$thumb_id = get_post_thumbnail_id( $slide_id );
		}

		$placeholder_img = PDT_URL . 'public/assets/img/placeholder.png';
		$placeholder_img = apply_filters( 'pdt_no_thumb_placeholder', $placeholder_img );

		if ( empty( $thumb_id ) && ! empty( $placeholder_img ) ) {
			$thumb_id = attachment_url_to_postid( $placeholder_img );
		}
		if ( $is_attachment ) {
			$thumb_id = $slide_id;
		}
		if ( ! empty( $thumb_id ) ) {
			$image_sizes          = isset( $product_thumb_setting['pdt_thumb_sizes'] ) ? $product_thumb_setting['pdt_thumb_sizes'] : 'large';
			$product_image_width  = isset( $product_thumb_setting['pdt_image_crop_size']['top'] ) ? $product_thumb_setting['pdt_image_crop_size']['top'] : '';
			$product_image_height = isset( $product_thumb_setting['pdt_image_crop_size']['right'] ) ? $product_thumb_setting['pdt_image_crop_size']['right'] : '';
			$product_image_crop   = isset( $product_thumb_setting['pdt_image_crop_size']['style'] ) ? $product_thumb_setting['pdt_image_crop_size']['style'] : '';
			$thumb_full_src       = wp_get_attachment_image_src( $thumb_id, 'full' );
			$thumb_full_src       = is_array( $thumb_full_src ) ? $thumb_full_src : array( '', '', '' );
			$image_src            = wp_get_attachment_image_src( $thumb_id, $image_sizes );
			$image_src            = is_array( $image_src ) ? $image_src : array( '', '', '' );

			if ( ( 'custom' === $image_sizes ) && ( ! empty( $product_image_width ) && $thumb_full_src[1] >= $product_image_width ) && ( ! empty( $product_image_height ) && $thumb_full_src[2] >= $product_image_height ) ) {
				$hard_crop = 'Hard-crop' === $product_image_crop ? true : false;

				$image = ta_pdt_resize( $thumb_full_src[0], $product_image_width, $product_image_height, $hard_crop );
				if ( $show_2x_image && ( $thumb_full_src[1] >= ( $product_image_width * 2 ) ) && $thumb_full_src[2] >= ( $product_image_height * 2 ) ) {
					$image_resize_2x_url = ta_pdt_resize( $thumb_full_src[0], $product_image_width * 2, $product_image_height * 2, $hard_crop );
				} elseif ( $show_2x_image && ( ( $product_image_width * 2 ) === $thumb_full_src[1] ) && ( $product_image_height * 2 ) === $thumb_full_src[2] ) {
					$image_resize_2x_url = $thumb_full_src[0];
				}
				$image_width  = $product_image_width;
				$image_height = $product_image_height;
			} else {
				$image        = ! empty( $image_src[0] ) ? $image_src[0] : $placeholder_img;
				$image_width  = ! empty( $image_src[1] ) ? $image_src[1] : 600;
				$image_height = ! empty( $image_src[2] ) ? $image_src[2] : 450;
			}
		} else {
			$image        = $placeholder_img;
			$image_width  = 600;
			$image_height = 450;
		}
		$pdt_image_attr = array(
			'src'        => $image,
			'2x_src'     => $image_resize_2x_url,
			'width'      => $image_width,
			'height'     => $image_height,
			'video'      => ! empty( $replace_thumb['video'] ) ? $replace_thumb['video'] : '',
			'audio'      => ! empty( $replace_thumb['audio'] ) ? $replace_thumb['audio'] : '',
			'aria_label' => ! empty( $replace_thumb['aria_label'] ) ? $replace_thumb['aria_label'] : 'feature_image',
		);

		return $pdt_image_attr;
	}


	/**
	 * Thumb Sized function
	 *
	 * @param  array $view_options options array.
	 * @param  int   $slide_id post id.
	 * @param  bool  $is_attachment type attachment.
	 * @return array
	 */
	public static function pdt_sized_pdtup_thumb( $view_options, $slide_id, $is_attachment = false ) {
		$thumb_id     = '';
		$image        = '';
		$image_height = '';
		$image_width  = '';
		if ( has_post_thumbnail( $slide_id ) ) {
			$thumb_id = get_post_thumbnail_id( $slide_id );
		}
		if ( $is_attachment ) {
			$thumb_id = $slide_id;
		}
		if ( ! empty( $thumb_id ) ) {
			$image_sizes          = isset( $view_options['pdt_pdtup_img_sizes'] ) ? $view_options['pdt_pdtup_img_sizes'] : 'medium';
			$product_image_width  = isset( $view_options['pdt_pdtup_image_crop_size']['top'] ) ? $view_options['pdt_pdtup_image_crop_size']['top'] : '';
			$product_image_height = isset( $view_options['pdt_pdtup_image_crop_size']['right'] ) ? $view_options['pdt_pdtup_image_crop_size']['right'] : '';
			$product_image_crop   = isset( $view_options['pdt_pdtup_image_crop_size']['style'] ) ? $view_options['pdt_pdtup_image_crop_size']['style'] : '';
			$thumb_full_src       = wp_get_attachment_image_src( $thumb_id, 'full' );
			$thumb_full_src       = is_array( $thumb_full_src ) ? $thumb_full_src : array( '', '', '' );
			$image_src            = wp_get_attachment_image_src( $thumb_id, $image_sizes );
			$image_src            = is_array( $image_src ) ? $image_src : array( '', '', '' );
			if ( ( 'custom' === $image_sizes ) && ( ! empty( $product_image_width ) && $thumb_full_src[1] >= $product_image_width ) && ( ! empty( $product_image_height ) && $thumb_full_src[2] >= $product_image_height ) ) {
				$hard_crop = 'Hard-crop' === $product_image_crop ? true : false;

				$image        = ta_pdt_resize( $thumb_full_src[0], $product_image_width, $product_image_height, $hard_crop );
				$image_width  = $product_image_width;
				$image_height = $product_image_height;
			} else {
				$image        = ! empty( $image_src[0] ) ? $image_src[0] : '';
				$image_width  = ! empty( $image_src[1] ) ? $image_src[1] : 600;
				$image_height = ! empty( $image_src[2] ) ? $image_src[2] : 450;
			}
		}
		return array(
			'src'    => $image,
			'width'  => $image_width,
			'height' => $image_height,
		);
	}
	/**
	 * Thumb Sized function
	 *
	 * @param array   $product_thumb_setting Thumbnails options array.
	 * @param integer $slide_id The slide/post ID.
	 *
	 * @return string
	 */
	public static function pdt_thumb_replace( $product_thumb_setting, $slide_id ) {
		$pdt_thumb_src_replace = isset( $product_thumb_setting['pdt_thumb_src_replace'] ) ? $product_thumb_setting['pdt_thumb_src_replace'] : '';

		if ( is_array( $pdt_thumb_src_replace ) ) {
			$image_src    = in_array( 'image', $pdt_thumb_src_replace, true ) ? true : false;
			$video_src    = in_array( 'video', $pdt_thumb_src_replace, true ) ? true : false;
			$audio_src    = in_array( 'audio', $pdt_thumb_src_replace, true ) ? true : false;
			$content_post = get_post( $slide_id );
			$content      = $content_post->post_content;

			$test          = preg_match_all(
				'((https?|ftp|gopher|telnet|file|notes|ms-help):' .
					'((//)|(\\\\))+[\w\d:#@%/;$()~_?\+-=\\\.&]*)',
				$content,
				$matches,
				PREG_PATTERN_ORDER
			);
			$first_content = 'image';

			if ( ! empty( $matches[0][0] ) ) {
				if ( strpos( $matches[0][0], '.jpg' ) || strpos( $matches[0][0], '.jpeg' ) || strpos( $matches[0][0], '.gif' ) || strpos( $matches[0][0], '.png' ) || strpos( $matches[0][0], '.eps' ) ) {
					$first_content = 'image';
				}
				if ( strpos( $matches[0][0], 'video' ) || strpos( $matches[0][0], 'youtube' ) || strpos( $matches[0][0], 'vimeo' ) || strpos( $matches[0][0], '.mp4' ) ) {
					$first_content = 'video';
				}
				if ( strpos( $matches[0][0], 'audio' ) || strpos( $matches[0][0], 'soundcloud' ) ) {
					$first_content = 'audio';
				}
			}
			$image_id = 'image' === $first_content ? self::pdt_get_img_from_post( $product_thumb_setting, $slide_id ) : '';
			$is_video = 'video' === $first_content ? self::pdt_get_video_from_post( $slide_id ) : '';
			$is_audio = 'audio' === $first_content ? self::pdt_get_audios_from_post( $slide_id ) : '';

			if ( $image_src && $video_src && $audio_src ) {
				$thumb_id    = $image_id;
				$video_thumb = $is_video;
				$audio_thumb = $is_audio;
			}
			if ( $image_src && $video_src ) {
				$thumb_id    = $image_id;
				$video_thumb = $is_video;
			}
			if ( $image_src && $audio_src ) {
				$thumb_id    = $image_id;
				$audio_thumb = $is_audio;
			}
			if ( $audio_src && $video_src ) {
				$video_thumb = $is_video;
				$audio_thumb = $is_audio;
			}
			if ( $image_src ) {
				$thumb_id = self::pdt_get_img_from_post( $product_thumb_setting, $slide_id );
			}
			if ( $video_src && ( empty( $matches[1][0] ) || ! $image_src ) ) {
				$video_thumb = self::pdt_get_video_from_post( $slide_id );
			}
			if ( $audio_src && empty( $video_thumb ) && ( empty( $matches[1][0] ) || ! $image_src ) ) {
				$audio_thumb = self::pdt_get_audios_from_post( $slide_id );
			}
		}
		$pdt_thumbs_replace_src = array(
			'id'         => ! empty( $thumb_id ) ? $thumb_id : '',
			'video'      => ! empty( $video_thumb ) ? $video_thumb : '',
			'audio'      => ! empty( $audio_thumb ) ? $audio_thumb : '',
			'aria_label' => ! empty( $first_content ) ? $first_content : '',
		);
		return $pdt_thumbs_replace_src;
	}

	/**
	 * Get first image from post content
	 *
	 * @since 1.0
	 * @param array  $product_thumb_setting Thumbnails options array.
	 * @param  number $slide_id Post id.
	 * @return mixed
	 */
	public static function pdt_get_img_from_post( $product_thumb_setting, $slide_id ) {
		$pdt_thumb_src = isset( $product_thumb_setting['pdt_thumb_src'] ) ? $product_thumb_setting['pdt_thumb_src'] : 'featured_image';
		$content_post  = get_post( $slide_id );
		$content       = $content_post->post_content;
		$images        = preg_match_all( '/<img[^>]* src=\"([^\"]*)\"[^>]*>/i', $content, $matches );
		if ( $images ) {
			$img_url  = 'last_img_content' === $pdt_thumb_src ? array_values( array_slice( $matches[1], -1 ) ) : $matches[1][0];
			$thumb_id = self::pdt_image_id_by_url( $img_url, $slide_id );
		}
		if ( ! empty( $thumb_id ) ) {
			return $thumb_id;
		}
	}

	/**
	 * Get first video from post content
	 *
	 * @since 1.0
	 * @param  number $slide_id Post id.
	 * @return mixed
	 */
	public static function pdt_get_video_from_post( $slide_id ) {

		$product = get_post( $slide_id );
		$content = do_shortcode( apply_filters( 'the_content', $product->post_content ) );
		$embeds  = apply_filters( 'pdt_get_post_video', get_media_embedded_in_content( $content ) );

		if ( empty( $embeds ) ) {
			return '';
		}

		// check what is the first embed containg video tag, youtube or vimeo.
		foreach ( $embeds as $embed ) {
			if ( strpos( $embed, 'video' ) || strpos( $embed, 'youtube' ) || strpos( $embed, 'vimeo' ) || strpos( $embed, '.mp4' ) ) {
				return $embed;
			}
		}
	}

	/**
	 * Get audio files from post content
	 *
	 * @param  number $slide_id Post id.
	 * @return mixed          Iframe.
	 */
	public static function pdt_get_audios_from_post( $slide_id ) {
		// For audio post type - grab.
		$product = get_post( $slide_id );
		$content = do_shortcode( apply_filters( 'ta_pdt_the_content', $product->post_content ) );
		$embeds  = apply_filters( 'pdt_get_post_audio', get_media_embedded_in_content( $content ) );
		if ( empty( $embeds ) ) {
			return '';
		}

		// check what is the first embed containg video tag, youtube or vimeo.
		foreach ( $embeds as $embed ) {
			if ( strpos( $embed, 'audio' ) || strpos( $embed, 'soundcloud' ) ) {
				return $embed;
			}
		}
	}

	/**
	 * Process all the post meta.
	 *
	 * @param object $product The selected post.
	 * @param array  $product_meta_fields The selected post meta to show.
	 * @param int    $visitor_count Number of visitor saw the post.
	 * @param string $meta_separator The post meta separator.
	 * @param string $is_table the table layout check.
	 * @return void
	 */
	public static function pdt_get_post_meta( $product, $product_meta_fields, $visitor_count, $meta_separator, $is_table ) {

		$meta_wrapper_start_tag = ! $is_table ? apply_filters( 'pdt_product_meta_wrapper_start', '<ul>' ) : '';
		$meta_wrapper_end_tag   = ! $is_table ? apply_filters( 'pdt_product_meta_wrapper_end', '</ul>' ) : '';
		echo wp_kses_post( $meta_wrapper_start_tag );
		$i = 0;
		foreach ( $product_meta_fields as $each_meta ) {
			$selected_meta      = isset( $each_meta['select_post_meta'] ) ? $each_meta['select_post_meta'] : '';
			$taxonomy_name      = isset( $each_meta['post_meta_taxonomy'] ) ? $each_meta['post_meta_taxonomy'] : '';
			$meta_date_format   = isset( $each_meta['post_meta_date_format'] ) ? $each_meta['post_meta_date_format'] : 'default';
			$custom_date_format = isset( $each_meta['pdt_custom_meta_date_format'] ) ? $each_meta['pdt_custom_meta_date_format'] : 'F j, Y';
			$word_per_minute    = isset( $each_meta['pdt_word_per_minute'] ) ? $each_meta['pdt_word_per_minute'] : 300;

			$meta_icon      = ! empty( $each_meta['select_meta_icon'] ) ? sprintf( '<i class="' . $each_meta['select_meta_icon'] . '"></i>' ) : '';
			$start_tag      = $is_table ? '<td class="ta-pdt-post-meta">' : '<li>';
			$end_tag        = $is_table ? '</td>' : '</li>';
			$meta_tag_start = apply_filters( 'pdt_product_meta_html_tag_start', $start_tag );
			$meta_tag_end   = apply_filters( 'pdt_product_meta_html_tag_end', $end_tag );
			$allowed_html   = array(
				'a'    => array(
					'href'  => array(),
					'title' => array(),
				),
				'i'    => array(
					'class' => array(),
					'id'    => array(),
				),
				'span' => array(
					'class' => array(),
					'id'    => array(),
				),
				'div'  => array(
					'class' => array(),
					'id'    => array(),
				),
			);

			switch ( $selected_meta ) {
				case 'date':
					if ( 0 < $i ) {
						?>
						<span class="meta_separator"><?php echo wp_kses_post( $meta_separator ); ?></span>
						<?php
					}
					echo wp_kses_post( $meta_tag_start );
					if ( 'default' === $meta_date_format ) {
						$product_date = esc_html( date_i18n( get_option( 'date_format' ), strtotime( get_the_date('d-m-Y', $product->get_id()) ) ) );
					} elseif ( 'time_ago' === $meta_date_format ) {
						$product_date = human_time_diff( date_i18n( 'U', strtotime( get_the_date('d-m-Y', $product->get_id()) ) ), current_time( 'timestamp' ) ) . esc_html__( ' ago', 'productify-pro' );
					} elseif ( 'custom' === $meta_date_format ) {
						$product_date = esc_html( date_i18n( $custom_date_format, strtotime( get_the_date('d-m-Y', $product->get_id()) ) ) );
					}
					?>
					<?php echo wp_kses( $meta_icon, $allowed_html ); ?>
					<time class="entry-date published updated">

						<?php echo wp_kses_post( $product_date ); ?></time>
					<?php
					echo wp_kses_post( $meta_tag_end );
					break;
				case 'taxonomy':
					if ( 'beside_meta' === $each_meta['pdt_meta_position'] ) {
						$term = self::pdt_taxonomy_terms( $taxonomy_name, $product->get_id(), $meta_icon );
						if ( ! empty( $term ) ) {
							if ( 0 < $i ) {
								?>
								<span class="meta_separator"><?php echo wp_kses_post( $meta_separator ); ?></span>
								<?php
							}
							echo wp_kses_post( $meta_tag_start );
							echo wp_kses_post( $term );
							echo wp_kses_post( $meta_tag_end );
						}
					}
					break;
				case 'comment_count':
					if ( ! empty( $product->comment_count ) ) {
						if ( 0 < $i ) {
							?>
							<span class="meta_separator"><?php echo wp_kses_post( $meta_separator ); ?></span>
							<?php
						}
						echo wp_kses_post( $meta_tag_start );
						?>
						<?php echo wp_kses( $meta_icon, $allowed_html ); ?>
						<a href="<?php esc_url( $product->guid ); ?>"><?php echo esc_html( $product->comment_count ); ?></a>
						<?php
						echo wp_kses_post( $meta_tag_end );
					} elseif ( $is_table ) {
						echo wp_kses_post( $meta_tag_start );
						echo ' ';
						echo wp_kses_post( $meta_tag_end );
					}
					break;
				case 'view_count':
					if ( 0 < $i ) {
						?>
						<span class="meta_separator"><?php echo wp_kses_post( $meta_separator ); ?></span>
						<?php
					}
					echo wp_kses_post( $meta_tag_start );
					?>
					<?php echo wp_kses( $meta_icon, $allowed_html ); ?> <span><?php echo esc_html( $visitor_count ); ?></span>
					<?php
					echo wp_kses_post( $meta_tag_end );
					break;
				case 'like':
					if ( 0 < $i ) {
						?>
						<span class="meta_separator"><?php echo wp_kses_post( $meta_separator ); ?></span>
						<?php
					}
					echo wp_kses_post( $meta_tag_start );
					?>
					<?php
					echo wp_kses(
						PDT_User_Like::get_pdt_likes_button( $product->get_id() ),
						array(
							'a'    => array(
								'href'           => true,
								'class'          => true,
								'data-nonce'     => true,
								'data-post-id'   => true,
								'data-iscomment' => true,
								'title'          => true,
							),
							'i'    => array( 'class' => true ),
							'span' => array( 'class' => true ),
						)
					);
					?>
					<?php
					echo wp_kses_post( $meta_tag_end );
					break;
			}
			++$i;
		} // End Foreach.
		echo wp_kses_post( $meta_wrapper_end_tag );
	}

	/**
	 * Pcp_get_post_meta_on_table_heading
	 *
	 * @param  mixed $product_meta_fields The selected post meta to show.
	 * @return void
	 */
	public static function pdt_get_post_meta_on_table_heading( $product_meta_fields ) {
		$i = 0;
		foreach ( $product_meta_fields as $each_meta ) {
			$selected_meta = isset( $each_meta['select_post_meta'] ) ? $each_meta['select_post_meta'] : '';
			$taxonomy_name = isset( $each_meta['post_meta_taxonomy'] ) ? $each_meta['post_meta_taxonomy'] : '';
			switch ( $selected_meta ) {
				case 'date':
					?>
					<th><?php esc_html_e( 'Date', 'productify-pro' ); ?></th>
					<?php
					break;
				case 'taxonomy':
					if ( 'beside_meta' === $each_meta['pdt_meta_position'] ) {
						?>
						<th><?php echo esc_html( $taxonomy_name ); ?></th>
						<?php
					}
					break;
				case 'comment_count':
					?>
					<th><?php esc_html_e( 'Comment', 'productify-pro' ); ?></th>
					<?php

					break;
				case 'view_count':
					?>
					<th><?php esc_html_e( 'View count', 'productify-pro' ); ?></th>
					<?php
					break;
				case 'like':
					?>
					<th><?php esc_html_e( 'Like', 'productify-pro' ); ?></th>
					<?php
					break;
			}
			++$i;
		} // End Foreach.
	}

	/**
	 * Process the taxonomy terms.
	 *
	 * @param array $taxonomy The taxonomy name to show terms from.
	 * @param mixed $id ids.
	 * @param mixed $meta_icon The meta icon html.
	 * @return statement
	 */
	public static function pdt_taxonomy_terms( $taxonomy, $id, $meta_icon = null ) {
		$terms = get_the_term_list( $id, $taxonomy, '', ', ' );
		if ( ! empty( $terms ) && ! is_wp_error( $terms ) ) {
			return $meta_icon . $terms;
		}
	}

	/**
	 * Show post thumb category.
	 *
	 * @param integer $product_id The post ID.
	 * @return statement.
	 */
	public static function pdt_product_thumb_taxonomy( $taxonomy, $id, $meta_icon = null ) {
		$terms = get_the_terms( $id, $taxonomy );

		if ( ! empty( $terms ) && ! is_wp_error( $terms ) ) {
			$first_term = reset( $terms ); // Get the first term

			if ( $first_term ) {
				return '<a href="' . get_tag_link( $first_term->term_id ) . '">' . $first_term->name . '</a>';
			}
		}
	}

	/**
	 * Show post tags with separator.
	 *
	 * @param integer $product_id The post ID.
	 * @return statement.
	 */
	public static function pdt_product_tags( $product_id ) {
		$product_tags = get_the_tags( $product_id );
		$separator    = ', ';
		$output       = '';
		if ( ! empty( $product_tags ) ) {
			foreach ( $product_tags as $tag ) {
				$output .= '<a href="' . get_tag_link( $tag->term_id ) . '">' . $tag->name . '</a>' . $separator;
			}
			return trim( $output, $separator );
		}
	}

	/**
	 * Maximum pages.
	 *
	 * @param int $total_post Number of total posts.
	 * @param int $product_per_page Posts per page.
	 *
	 * @return void
	 */
	public static function pdt_max_pages( $total_post, $product_per_page ) {
		if ( ! $total_post ) {
			return;
		}
		$max_num_pages = ceil( $total_post / $product_per_page );
		return (int) $max_num_pages;
	}

	/**
	 * Post per page.
	 *
	 * @param int $limit Post Limit.
	 * @param int $product_per_page post per page.
	 * @param int $page paged number.
	 *
	 * @return int
	 */
	public static function pdt_product_per_page( $limit, $product_per_page, $page ) {
		$limit               = ( empty( $limit ) || '-1' === $limit ) ? 10000000 : $limit;
		$offset              = (int) $product_per_page * ( $page - 1 );
		$final_post_per_page = $product_per_page;
		if ( intval( $product_per_page ) > $limit - $offset ) {
			$final_post_per_page = $limit - $offset;
		}
		return $final_post_per_page;
	}

	/**
	 * Pagination last page post
	 *
	 * @param [type] $limit total post limit.
	 * @param [type] $product_per_page post per page.
	 * @param [type] $total_page last post page.
	 *
	 * @return int.
	 */
	public static function pdt_last_page_post( $limit, $product_per_page, $total_page ) {
		$limit              = ( empty( $limit ) || '-1' === $limit ) ? 10000000 : $limit;
		$offset             = $product_per_page * ( $total_page - 1 );
		$pdt_last_page_post = $limit - $offset;
		return $pdt_last_page_post;
	}

	/**
	 * Get view option from view ID
	 *
	 * @param string $pdt_gl_id ID of custom field.
	 *
	 * @return array
	 */
	public static function view_options( $pdt_gl_id ) {
		if ( ! $pdt_gl_id ) {
			return;
		}
		$view_options = get_post_meta( $pdt_gl_id, 'ta_pdt_view_options', true );
		return $view_options;
	}

	/**
	 * Get value of a setting from global settings array
	 *
	 * @param string     $field        The full name of setting to get value.
	 * @param array      $array_to_get Array to get values of wanted setting.
	 * @param mixed|null $assign       The value to assign if setting is not found.
	 */
	public static function pdt_metabox_value( $field, $array_to_get = null, $assign = null ) {
		global $pdt_gl_id;
		if ( empty( $array_to_get ) ) {
			$array_to_get = self::view_options( $pdt_gl_id );
		}
		return isset( $array_to_get[ $field ] ) ? $array_to_get[ $field ] : $assign;
	}

	/**
	 * Convert the slug text to a name.
	 *
	 * @param string $slug The slug string.
	 * @return string
	 */
	public static function slug_string_to_name( $slug ) {
		$slug = preg_replace( '/[_\-]+/', ' ', $slug );

		return ucwords( $slug );
	}

	/**
	 * Custom Template locator .
	 *
	 * @param  mixed $template_name template name .
	 * @param  mixed $template_path template path .
	 * @param  mixed $default_path default path .
	 * @return string
	 */
	public static function pdt_locate_template( $template_name, $template_path = '', $default_path = '' ) {
		if ( ! $template_path ) {
			$template_path = 'productify-pro/templates';
		}

		if ( ! $default_path ) {
			$default_path = PDT_PATH . 'public/templates/';
		}
		$template = locate_template( trailingslashit( $template_path ) . $template_name );
		// Get default template.
		if ( ! $template ) {
			$template = $default_path . $template_name;
		}
		// Return what we found.
		return $template;
	}
}
